@@ -18,6 +18,7 @@ use arrow::array::StringArray;
 use arrow::array::Time32SecondArray;
 use arrow::array::TimestampMicrosecondArray;
 use arrow::array::TimestampMillisecondArray;
+use arrow::array::TimestampNanosecondArray;
 use arrow::array::UInt16Array;
 use arrow::array::UInt32Array;
 use arrow::array::UInt64Array;
@@ -151,10 +152,7 @@ pub(crate) fn get_token_rows<'a, 'b>(
                     rowindex += 1;
                 }
             }
-            arrow::datatypes::DataType::Timestamp(
-                arrow::datatypes::TimeUnit::Millisecond,
-                None,
-            ) => {
+            arrow::datatypes::DataType::Timestamp(arrow::datatypes::TimeUnit::Millisecond, _) => {
                 let ba = col
                     .as_any()
                     .downcast_ref::<TimestampMillisecondArray>()
@@ -176,10 +174,7 @@ pub(crate) fn get_token_rows<'a, 'b>(
                     rowindex += 1;
                 }
             }
-            arrow::datatypes::DataType::Timestamp(
-                arrow::datatypes::TimeUnit::Microsecond,
-                None,
-            ) => {
+            arrow::datatypes::DataType::Timestamp(arrow::datatypes::TimeUnit::Microsecond, _) => {
                 let ba = col
                     .as_any()
                     .downcast_ref::<TimestampMicrosecondArray>()
@@ -200,9 +195,53 @@ pub(crate) fn get_token_rows<'a, 'b>(
                     rowindex += 1;
                 }
             }
+            arrow::datatypes::DataType::Timestamp(arrow::datatypes::TimeUnit::Nanosecond, _) => {
+                let ba = col
+                    .as_any()
+                    .downcast_ref::<TimestampNanosecondArray>()
+                    .unwrap();
+
+                let mut rowindex = 0;
+                for val in ba.iter() {
+                    token_rows[rowindex].push(match val {
+                        Some(vs) => {
+                            if vs < 0 {
+                                ColumnData::DateTime2(None)
+                            } else {
+                                to_col_dt((unix_min + StdDuration::from_nanos(vs as u64)).to_sql())
+                            }
+                        }
+                        None => ColumnData::DateTime2(None),
+                    });
+                    rowindex += 1;
+                }
+            }
             arrow::datatypes::DataType::Null => {
-                for rowindex in 0..rows {
-                    token_rows[rowindex].push(ColumnData::I32(None));
+                if coltype == &ColumnType::BigVarChar
+                    || coltype == &ColumnType::Text
+                    || coltype == &ColumnType::NVarchar
+                    || coltype == &ColumnType::NChar
+                    || coltype == &ColumnType::BigChar
+                    || coltype == &ColumnType::NText
+                {
+                    for rowindex in 0..rows {
+                        token_rows[rowindex].push(ColumnData::String(None));
+                    }
+                } else if coltype == &ColumnType::Datetime
+                    || coltype == &ColumnType::Datetimen
+                    || coltype == &ColumnType::Datetime4
+                {
+                    for rowindex in 0..rows {
+                        token_rows[rowindex].push(ColumnData::DateTime(None));
+                    }
+                } else if coltype == &ColumnType::Bit || coltype == &ColumnType::Bitn {
+                    for rowindex in 0..rows {
+                        token_rows[rowindex].push(ColumnData::Bit(None));
+                    }
+                } else {
+                    for rowindex in 0..rows {
+                        token_rows[rowindex].push(ColumnData::I32(None));
+                    }
                 }
             }
             arrow::datatypes::DataType::UInt8 => {
